package zero1hd.polyjet.audio;

import java.io.IOException;
import java.security.InvalidParameterException;

import javax.sound.sampled.AudioFormat;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.audio.Music;
import com.badlogic.gdx.files.FileHandle;

import zero1hd.wavedecoder.WavDecoder;
import zero1hd.wavedecoder.WavInfo;

public class WavAudioData implements AudioData {
	private int readWindowSize = 1024;
	private AudioFormat format;
	int readIndex;
	Music playbackMusic;
	WavDecoder decoder;
	WavInfo wavinfo;
	
	@Override
	public void readIndexUpdate() {
		readIndex = (int) (playbackMusic.getPosition() * wavinfo.getSampleRate() / readWindowSize);
	}
	
	@Override
	public int getReadIndex() {
		return readIndex;
	}
	
	public void setInfo(WavInfo wavinfo) {
		this.wavinfo = wavinfo;
	}
	
	@Override
	public void setAudioFile(FileHandle audioFileHandler) {
		reset();
		try {
			decoder.setAudioFile(wavinfo);
		} catch (InvalidParameterException | IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		playbackMusic = Gdx.audio.newMusic(audioFileHandler);
		format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, (float) wavinfo.getSampleRate(), 16, wavinfo.getChannels(), wavinfo.getChannels()*2, (float) wavinfo.getSampleRate(), false);
	}
	
	@Override
	public void reset() {
		if (playbackMusic != null) {
			playbackMusic.stop();
			playbackMusic.dispose();
			playbackMusic = null;
		}
	}
	
	@Override
	public int getReadWindowSize() {
		return readWindowSize;
	}
	
	@Override
	public Music getPlaybackMusic() {
		return playbackMusic;
	}

	@Override
	public int readSamples(float[] samples) {
		return decoder.readSamples(samples);
	}

	@Override
	public AudioFormat getFormat() {
		return format;
	}
}
