package zero1hd.polyjet.ui.pages;

import java.io.File;
import java.io.FilenameFilter;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.polyjet.Polyjet;
import zero1hd.polyjet.screens.MainMenu;
import zero1hd.polyjet.ui.builders.MusicSelectable;
import zero1hd.polyjet.ui.windows.LoadingWindow;

public class MusicSelectionPage extends Page {
	private volatile Table musicChoices;
	Preferences musicFileAnnotation;
	private Polyjet core;
	private volatile ScrollPane musicChoiceScroller;
	
	private volatile LoadingWindow loadingWindow;
	
	public MusicSelectionPage(final Polyjet core, final MainMenu mainMenu) {
		super("Select music", core.defaultSkin);
		this.core = core;
		
		musicFileAnnotation = Gdx.app.getPreferences("music_file_annotation");
		
		
		TextButton back = new TextButton("Back", core.defaultSkin);
		back.setPosition(getWidth()-back.getWidth()-15f, getHeight()-back.getHeight()-15f);
		back.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				core.setScreen(mainMenu);
			}
		});
		addActor(back);
		
		loadingWindow = new LoadingWindow(core.defaultSkin, "tinted", true, core.assetManager, core.prefs.getFloat("fx vol"));
		loadingWindow.setPosition((getWidth()-loadingWindow.getWidth())/2f, (getHeight()-loadingWindow.getHeight())/2f);
		addActor(loadingWindow);
		loadingWindow.playOpenSound();
		loadingWindow.setMovable(false);
		loadingWindow.setModal(true);
		
		musicChoices = new Table();
		musicChoices.defaults().pad(10f);
		musicChoiceScroller = new ScrollPane(musicChoices);
		musicChoiceScroller.setScrollingDisabled(false, true);
		musicChoiceScroller.setSize(getWidth(), getHeight()*0.85f);
		
		musicChoiceScroller.debug();
		addActor(musicChoiceScroller);
		
		loadingWindow.toFront();
	}
	
	@Override
	public void act(float delta) {
		
		super.act(delta);
	}
	
	public void beginMusicSearch() {
		new Thread(new Runnable() {
			
			@Override
			public void run() {
				FileHandle[] musicFiles = new FileHandle(core.prefs.getString("music dir")).list(new FilenameFilter() {
					@Override
					public boolean accept(File dir, String name) {
						if (name.endsWith("mp3") || name.endsWith("wav")) {
							return true;
						}
						return false;
					}
				});
				
				if (musicFiles.length != 0) {
					for (int music = 0; music < musicFiles.length; music++) {
						final MusicSelectable selectable = new MusicSelectable(musicFiles[music], musicFileAnnotation, core.defaultSkin, core.assetManager.get("defaultCover.png", Texture.class));
						Gdx.app.postRunnable(new Runnable() {
							@Override
							public void run() {
								Gdx.app.debug("Music Search Thread", "Finished loading: " + selectable.getName());
								musicChoices.add(selectable).prefSize(0.3f*getWidth(), 0.8f*getHeight());
								selectable.addInfoToPanel(musicChoiceScroller, 0.28f*getWidth());
							}
						});
						System.out.println(music);
						int prog = (int) (100f*music/(musicFiles.length-1f));
						loadingWindow.setProgress(prog);
					}
					
					loadingWindow.remove();
					
				} else {
				}
			}
		}).start();
		
		
	}
}
