package zero1hd.polyjet.ui.stages;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input.Keys;
import com.badlogic.gdx.scenes.scene2d.Actor;
import com.badlogic.gdx.scenes.scene2d.InputEvent;
import com.badlogic.gdx.scenes.scene2d.InputListener;
import com.badlogic.gdx.scenes.scene2d.Stage;
import com.badlogic.gdx.scenes.scene2d.ui.CheckBox;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.scenes.scene2d.ui.TextButton;
import com.badlogic.gdx.scenes.scene2d.ui.Window;
import com.badlogic.gdx.scenes.scene2d.utils.ChangeListener;

import zero1hd.polyjet.Polyjet;
import zero1hd.polyjet.audio.AudioAnalyzer;
import zero1hd.polyjet.audio.WavAudioData;
import zero1hd.polyjet.screens.MainMenu;
import zero1hd.polyjet.ui.windows.BeatViewer;
import zero1hd.polyjet.ui.windows.FPSWindow;
import zero1hd.polyjet.ui.windows.GraphWindow;
import zero1hd.polyjet.ui.windows.MusicController;
import zero1hd.polyjet.ui.windows.MusicSelector;
import zero1hd.polyjet.ui.windows.VolumeWindow;

public class CreativeStage extends Stage {
	WavAudioData audioWrapper;
	MusicController musicPlayBackControls;
	MusicSelector musicSelector;
	FPSWindow fpsViewer;
	BeatViewer beatViewer;
	GraphWindow graphViewer;
	VolumeWindow volumeWindow;
	
	AudioAnalyzer analyzer;
	Window toolbox;
	Polyjet core;
	boolean postAnalysisComplete;
	
	public CreativeStage(final Polyjet core, final MainMenu mainMenu) {
		this.core = core;
		analyzer = new AudioAnalyzer();
		audioWrapper = new WavAudioData();
		musicPlayBackControls = new MusicController(core.defaultSkin, audioWrapper);
		musicSelector = new MusicSelector("Select Audio File", core.defaultSkin, core.prefs.getString("music dir"), "default");
		musicSelector.postInit();
		musicSelector.refresh();
		fpsViewer = new FPSWindow("FPS", core.defaultSkin);
		beatViewer = new BeatViewer("Beat", core.defaultSkin, core, analyzer);
		graphViewer = new GraphWindow("Peak Values", core.defaultSkin);
		volumeWindow = new VolumeWindow("Volume adjustments", core.defaultSkin, core.prefs);
		
		//Back button
		TextButton backButton = new TextButton("Back", core.defaultSkin);
		backButton.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				dispose();
				core.setScreen(mainMenu);
			}
		});
		backButton.setPosition(10, Gdx.graphics.getHeight()-backButton.getHeight()-10);
		addActor(backButton);
		
		
		toolbox = new Window("Tools", core.defaultSkin);
		toolbox.defaults().pad(5f);
		
		Table toolboxToolSet = new Table(core.defaultSkin);
		toolboxToolSet.defaults().space(5f);
		
		final CheckBox musicSelectorCheckbox = new CheckBox(" Music selector", core.defaultSkin);
		musicSelectorCheckbox.addListener(new ChangeListener() {

			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (musicSelectorCheckbox.isChecked()) {
					addActor(musicSelector);
				} else {
					musicSelector.remove();
				}
			}
		});
		toolboxToolSet.add(musicSelectorCheckbox);
		toolboxToolSet.row();
		
		final CheckBox musicPlayBackCheckbox = new CheckBox(" Playback controls", core.defaultSkin);
		musicPlayBackCheckbox.addListener(new ChangeListener() {

			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (musicPlayBackCheckbox.isChecked()) {
					addActor(musicPlayBackControls);
				} else {
					musicPlayBackControls.remove();
				}
			}
		});
		toolboxToolSet.add(musicPlayBackCheckbox);
		toolboxToolSet.row();
		
		final CheckBox beatViewerCheckbox = new CheckBox(" Beat viewer", core.defaultSkin);
		beatViewerCheckbox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (beatViewerCheckbox.isChecked()) {
					addActor(beatViewer);
				} else {
					beatViewer.remove();
				}
			}
		});
		toolboxToolSet.add(beatViewerCheckbox);
		toolboxToolSet.row();
		
		final CheckBox fpsViewerCheckbox = new CheckBox(" FPS", core.defaultSkin);
		fpsViewerCheckbox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (fpsViewerCheckbox.isChecked()) {
					addActor(fpsViewer);
				} else {
					fpsViewer.remove();
				}
			}
		});
		toolboxToolSet.add(fpsViewerCheckbox);
		toolboxToolSet.row();
		
		final CheckBox peakGraphCheckbox = new CheckBox(" Peak Graph", core.defaultSkin);
		peakGraphCheckbox.addListener(new ChangeListener() {
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (peakGraphCheckbox.isChecked()) {
					addActor(graphViewer);
				} else {
					graphViewer.remove();
				}
				
			}
		});
		toolboxToolSet.add(peakGraphCheckbox);
		toolboxToolSet.row();
		
		final CheckBox volumeMixerCheckbox = new CheckBox(" Volume", core.defaultSkin);
		volumeMixerCheckbox.addListener(new ChangeListener() {
			
			@Override
			public void changed(ChangeEvent event, Actor actor) {
				if (volumeMixerCheckbox.isChecked()) {
					addActor(volumeWindow);
				} else {
					volumeWindow.remove();
				}
			}
		});
		
		toolboxToolSet.add(volumeMixerCheckbox);
		
		ScrollPane scroller = new ScrollPane(toolboxToolSet, core.defaultSkin);
		toolbox.add(scroller).expand().fill();
		toolbox.setSize(300, 300);
		addActor(toolbox);
		
		addListener(new InputListener() {
			@Override
			public boolean keyUp(InputEvent event, int keycode) {
				if (keycode == Keys.ENTER || keycode == Keys.ESCAPE) {
					setKeyboardFocus(null);
				}
				return super.keyUp(event, keycode);
			}
		});
	}

	@Override
	public void act(float delta) {
		if (musicSelector.isConfirmed()) {
			audioWrapper.setAudioFile(musicSelector.getSelectedMusic());
			audioWrapper.getPlaybackMusic().setVolume(core.prefs.getFloat("music vol")/100f);
			analyzer.resetVars();
			analyzer.startAnalyticalThread(audioWrapper);
			musicPlayBackControls.setMusicReady(false);
			beatViewer.setMusicReady(false);
			graphViewer.setData(analyzer.getBassPeaks(), analyzer.getUMPeaks());
			postAnalysisComplete = false;
		}
		
		if (!postAnalysisComplete && analyzer.containsData) {
			postAnalysisComplete = true;
			musicPlayBackControls.setMusicReady(true);
			beatViewer.setMusicReady(true);

		}
		
		if (analyzer.audiofile != null) {
			if (volumeWindow.isUpdateRequired()) {
				analyzer.audiofile.getPlaybackMusic().setVolume(core.prefs.getFloat("music vol")/100f);
			}
			
			analyzer.audiofile.readIndexUpdate();
			beatViewer.setSongIndex(analyzer.getReadIndex());
			graphViewer.getGraph().setAudioDataIndex(analyzer.getReadIndex());
		}
		super.act(delta);
	}
	
	@Override
	public void dispose() {
		analyzer.resetVars();
		analyzer.shrinkData();
		audioWrapper.reset();
		super.dispose();
	}
	
}
